const fs = require('fs-extra')
const path = require('path')
const ora = require('ora')
const kebabCase = require('lodash.kebabcase')
const { print, run } = require('@toptal/davinci-cli-shared')

const skeleton = require('../utils/skeleton')

const checkPrerequisites = async (destDir, appName) => {
  const spinner = ora('Checking prerequisites...').start()

  // look for env resolvers
  try {
    await run('direnv', ['version'])
  } catch (err) {
    spinner.warn()
    print.yellow(
      "warning: You don't have `direnv` for resolving env variables! Expect other approach is used."
    )
  }

  // check if app already exists
  const doesAppExist = await fs.exists(destDir)

  if (doesAppExist) {
    spinner.fail()
    throw new Error(`Application with name ${appName} already exists!`)
  }

  spinner.succeed('Prerequisites checked!')
}

const copyApplicationSkeleton = async (destDir, currentDavinciRunningDir) => {
  const skeletonPath = require.resolve(
    '@toptal/davinci-skeleton',
    {
      paths: [currentDavinciRunningDir]
    }
  )
  const appDir = path.join(skeletonPath, '../..')

  const spinner = ora('Copying skeleton...').start()

  try {
    await skeleton.copyAllFiles(appDir, destDir)
    await skeleton.copyGithub(destDir)
    await skeleton.copyNpmrc(destDir)
    await skeleton.copyTemplatePackageJsonFile(destDir)
    await skeleton.copyTemplateGitIgnoreFile(destDir)

    // remove bin directory
    await fs.remove(path.join(destDir, 'bin'))

    spinner.succeed('Skeleton copied!')
  } catch (err) {
    spinner.fail()
    throw err
  }
}

const modifyPackageJson = async (destDir, appName) => {
  const spinner = ora('Modifiying package.json...').start()

  try {
    const config = skeleton.readPackageJson(destDir)

    config.name = `@toptal/${appName}`
    config.homepage = `https://github.com/toptal/${appName}/tree/master#readme`

    delete config.gitHead

    delete config.scripts['prepack']
    delete config.scripts['postpack']
    delete config.scripts['build:package']
    delete config.scripts['prepublishOnly']

    delete config.publishConfig
    config.license = 'SEE LICENSE IN LICENSE.MD'

    config.version = '0.0.1'

    skeleton.writePackageJson(destDir, config)

    spinner.succeed('package.json modified!')
  } catch (err) {
    spinner.fail()
    throw err
  }
}

const installDependencies = async destDir => {
  const spinner = ora('Installing dependencies... ').start()

  // list of dependencies to be added to a new project
  const dependencies = [
    '@toptal/picasso-provider',
    '@toptal/picasso',
    '@topkit/apollo-client',
    'react',
    'react-dom',
    'styled-components',
    'typescript',
    '@apollo/client',
    'graphql'
  ]

  const devDependencies = [
    '@toptal/davinci',
    '@types/react',
    '@types/react-dom'
  ]

  try {
    await run('yarn', ['add', ...dependencies], { cwd: destDir })
    await run('yarn', ['add', '-D', ...devDependencies], { cwd: destDir })
    spinner.succeed('Dependencies installed!')
  } catch (err) {
    spinner.fail()
    throw new Error(err)
  }
}

const createCommand = async appName => {
  print.green('Creating', appName, '...')

  const currentRunningDir = process.cwd()
  const currentDavinciRunningDir = __dirname
  const destDir = path.join(currentRunningDir, appName)

  try {
    // check if all apps are installed, directory doesn't exists
    await checkPrerequisites(destDir, appName)

    // copy skeleton to destination directory
    await copyApplicationSkeleton(destDir, currentDavinciRunningDir)

    // change package.json
    await modifyPackageJson(destDir, appName)

    // run yarn
    await installDependencies(destDir)

    print.green('Application has been created.')
  } catch (err) {
    print.red(err)
    process.exit(1)
  }
}

const newCommandCreator = {
  command: 'new <appName>',
  description: 'Create new application',
  action: async ({ appName }) => {
    const formattedAppName = kebabCase(appName)

    return createCommand(formattedAppName)
  }
}

module.exports = newCommandCreator
